# 
# Type: mysql < xdb_model.sql
# 
# DDL for creating the xdb database from MySQL
#
# DROP DATABASE MYXDB_2003;
 CREATE DATABASE MYXDB_2003;
USE MYXDB_2003;
#DROP TABLE PC_SEQUENCE;
#DROP TABLE ATTR_SEQUENCE;
#DROP TABLE NS_SEQUENCE;
#DROP TABLE X_DOCUMENT;
#DROP TABLE X_ELEMENT;
#DROP TABLE X_ATTRIBUTE;
#DROP TABLE X_NAMESPACE;
#DROP TABLE X_PI;
#DROP TABLE X_COMMENT;
#DROP TABLE X_TEXT;
#DROP TABLE NEXT_OID;
#DROP TABLE XDB_COLLECTION;

# PC_SEQUENCE
# parent<->child relationship
CREATE TABLE PC_SEQUENCE
              (PARENT_NODE_OID BIGINT NOT NULL,
               PARENT_NODE_KIND VARCHAR(10) NOT NULL,
               CHILD_NODE_OID BIGINT NOT NULL,
               CHILD_NODE_KIND VARCHAR(10) NOT NULL);

# ATTR_SEQUENCE
# element<->attribute relationship
CREATE TABLE ATTR_SEQUENCE
              (X_ELEMENT_OID BIGINT NOT NULL,
               FOREIGN KEY (X_ELEMENT_OID) REFERENCES X_ELEMENT(X_ELEMENT_OID),
               X_ATTRIBUTE_OID BIGINT NOT NULL,
               FOREIGN KEY (X_ATTRIBUTE_OID) REFERENCES X_ATTRIBUTE(X_ATTRIBUTE_OID));

# NS_SEQUENCE
# element->namespace relationship
CREATE TABLE NS_SEQUENCE
              (X_ELEMENT_OID BIGINT NOT NULL,
               FOREIGN KEY (X_ELEMENT_OID) REFERENCES X_ELEMENT(X_ELEMENT_OID),
               X_NAMESPACE_OID BIGINT NOT NULL,
               FOREIGN KEY (X_NAMESPACE_OID) REFERENCES X_NAMESPACE(X_NAMESPACE_OID));

# DOCUMENT
# CHILDREN references PC_SEQUENCE(PARENT_NODE_OID).
# node-kind: "document"
CREATE TABLE X_DOCUMENT
              (X_DOCUMENT_OID BIGINT NOT NULL,
               BASE_URI LONG VARCHAR,
               STRING_VALUE LONG VARCHAR,
               XDB_COLLECTION_OID BIGINT,
               FOREIGN KEY (XDB_COLLECTION_OID) REFERENCES XDB_COLLECTION(XDB_COLLECTION_OID),
               PRIMARY KEY (X_DOCUMENT_OID));

# ELEMENT
# node-kind: "element"
# PARENT and CHILDREN reference PC_SEQUENCE.
# ATTRIBUTES references ATTR_SEQUENCE.
# TBD: typed-value,declaration,type
CREATE TABLE X_ELEMENT
              (X_ELEMENT_OID BIGINT NOT NULL, 
               NAMESPACE_URI LONG VARCHAR, 
               LOCAL_NAME VARCHAR(255),
               BASE_URI LONG VARCHAR,
               STRING_VALUE LONG VARCHAR,
               UNIQUE_ID VARCHAR(255),
               PRIMARY KEY (X_ELEMENT_OID));

# ATTRIBUTE
# node-kind: "attribute"
# PARENT reference ATTR_SEQUENCE.
# TBD: typed-value,declaration,type
CREATE TABLE X_ATTRIBUTE
              (X_ATTRIBUTE_OID BIGINT NOT NULL,
               NAMESPACE_URI LONG VARCHAR, 
               LOCAL_NAME VARCHAR(255),
               STRING_VALUE LONG VARCHAR,
               PRIMARY KEY (X_ATTRIBUTE_OID));

# NAMESPACE
# node-kind: "namespace"
CREATE TABLE X_NAMESPACE
              (X_NAMESPACE_OID BIGINT NOT NULL,
               NAMESPACE_URI LONG VARCHAR, 
               LOCAL_NAME VARCHAR(255),
               STRING_VALUE LONG VARCHAR,
               PRIMARY KEY (X_NAMESPACE_OID));

# PROCESSING INSTRUCTIONS
# node-kind: "processing-instruction"
# PARENT reference PC_SEQUENCE(either ELEMENT or DOCUMENT).
CREATE TABLE X_PI
              (X_PI_OID BIGINT NOT NULL,
               NAMESPACE_URI LONG VARCHAR, 
               LOCAL_NAME VARCHAR(255),
               STRING_VALUE LONG VARCHAR,
               PRIMARY KEY (X_PI_OID));

# COMMENT
# node-kind: "comment"
# PARENT reference PC_SEQUENCE(either ELEMENT or DOCUMENT).
CREATE TABLE X_COMMENT
              (X_COMMENT_OID BIGINT NOT NULL,
               STRING_VALUE LONG VARCHAR,
               PRIMARY KEY (X_COMMENT_OID));

# TEXT
# node-kind: "text"
# PARENT reference PC_SEQUENCE(either ELEMENT or DOCUMENT).
CREATE TABLE X_TEXT
              (X_TEXT_OID BIGINT NOT NULL,
               STRING_VALUE LONG VARCHAR,
               PRIMARY KEY (X_TEXT_OID));


# XML:DB Collection: "collection"
# Note that this is not part of the model.
CREATE TABLE XDB_COLLECTION
              (XDB_COLLECTION_OID BIGINT NOT NULL,
               NAME VARCHAR(255) NOT NULL,
               PARENT BIGINT NOT NULL,
               FOREIGN KEY (PARENT) REFERENCES XDB_COLLECTION(XDB_COLLECTION_OID),
               PRIMARY KEY (XDB_COLLECTION_OID));

# Root Collection
INSERT INTO XDB_COLLECTION VALUES (0,'root',-1);
               
# NEXT_OID
# Stores next OIDs to be used when inserting a new row
CREATE TABLE NEXT_OID
              (NEXT_OID_OID INT NOT NULL,
               X_DOCUMENT_OID BIGINT NOT NULL,
               X_NAMESPACE_OID BIGINT NOT NULL,
               X_ELEMENT_OID BIGINT NOT NULL,
               X_ATTRIBUTE_OID BIGINT NOT NULL,
               X_PI_OID BIGINT NOT NULL,
               X_COMMENT_OID BIGINT NOT NULL,
               X_TEXT_OID BIGINT NOT NULL,
               XDB_COLLECTION_OID BIGINT NOT NULL);

INSERT INTO NEXT_OID VALUES(0,0,0,0,0,0,0,0,1);

