/*
 * (C) Copyright Bora Software Inc. 2002, 2003 - All Rights Reserved.
 *
 * $Id: XElement.java,v 1.9 2002/10/20 03:01:35 kboo Exp $
 */
package com.bs.xdbms.datamodel;

import java.util.Vector;

public class XElement extends XNode {
 
  private XSQName xsqname; // expanded-QName

  private XElement() {
  }

  public XElement(String nsURI, String name) {
    nodeKind = "element";
    xsqname = new XSQName(nsURI,name);
    children = new Vector();
    attributes = new Vector();
    namespaces = new Vector();
  }

  public String getNamespaceURI() {
    return xsqname.getNamespaceURI();
  }

  public String getLocalName() {
    return xsqname.getLocalName();
  }

  public String getQName() {
    if (getPrefix() != null)
      return getPrefix() + ":" + getLocalName();
    else
      return getLocalName();
  }

  public String getPrefix() {
    XNamespace ns;
    for (int i=0; i<namespaces.size(); i++) {
      ns = (XNamespace)namespaces.elementAt(i);
      if (ns.getURI().equals(getNamespaceURI()))
        return ns.getPrefix();
    }
    return null;
  }

  public void setBaseURI(String anyURI) {
    baseURI = anyURI;
  }

  public void setUniqueID(String xsID) {
    uniqueID = xsID;
  }

  public void appendChild(XNode node) throws XModelException {
    String nk = node.getNodeKind();
    if (nk.equals("element")
        || nk.equals("processing-instruction")
        || nk.equals("text")
        || nk.equals("comment")) {
      children.addElement(node);
      node.setParent(this);
    }
    else
      throw new XModelException("Node kind: " + nk + " cannot be added to the Element node.");
  }

  public void addAttribute(XAttribute attr) {
    attributes.addElement(attr);
    attr.setParent(this);
  }

  public void addNamespace(XNamespace ns) {
    namespaces.addElement(ns);
  }
}
