/*
 * (C) Copyright Bora Software Inc. 2002, 2003 - All Rights Reserved.
 *
 * $Id: DOMToModel.java,v 1.4 2002/10/20 03:01:35 kboo Exp $
 */

package com.bs.xdbms.persistence;

import com.bs.xdbms.datamodel.*;
import org.w3c.dom.*;

import java.util.*;

public class DOMToModel {

  private Hashtable xnamespaces = new Hashtable();
  private String baseURI; // Document's base URI

  public DOMToModel(String baseURI) {
    this.baseURI = baseURI;
  }

  public XDocument convert(Document doc)  throws XModelException {
    if (doc == null)
      return null;

    XDocument xdoc = new XDocument();
    xdoc.setBaseURI(baseURI);
    // TBD - document.getDoctype() to retrieve public/system id
    XElement xel = convert(doc.getDocumentElement(),xdoc);
    xdoc.appendChild(xel);
    return xdoc;
  }

  private XElement convert(Element el, XDocument xdoc)  throws XModelException {
    String nsURI = el.getNamespaceURI();
    String localName = el.getLocalName();
    String rawName = el.getNodeName();

    //System.out.println("Node name is: " + el.getNodeName());
    //System.out.println("Converting DOM Element: " + nsURI + "/" + localName);

    XElement xel = xdoc.createElement(nsURI,localName);
    xel.setBaseURI(baseURI);
    NamedNodeMap nnm = el.getAttributes();

    Node attr;
    String attrNsURI;
    String attrLocalName;
    String attrRawName;
    String attrValue;
    XAttribute xattr;
    for (int i=0; i<nnm.getLength(); i++) {
      attr = nnm.item(i);
      attrNsURI = attr.getNamespaceURI();
      attrLocalName = attr.getLocalName();
      attrRawName = attr.getNodeName();
      attrValue = attr.getNodeValue();
      
      //System.out.println("Attribute uri is: " + attrNsURI);
      //System.out.println("Attribute local name is: " + attrLocalName);
      //System.out.println("Attribute raw name is: " + attrRawName);
      //System.out.println("Attribute value is: " + attrValue);

      // Anything that starts with "xmlns:" is not an attribute.
      if (!attrRawName.startsWith("xmlns:")) {
        xattr = xdoc.createAttribute(attrNsURI,attrLocalName,attrValue);
        xel.addAttribute(xattr);
      }
    }

    // Add a namespace
    XNamespace xns = getNamespace(nsURI,rawName);
    if (xns != null)
      xel.addNamespace(xns);

    // Process children

    Node child = el.getFirstChild();
    short type;
    XNode xnode = null;
    while (child != null) {
      //xnode = convert(child,xdoc);
      type = child.getNodeType();
      switch (type) {
        case Node.ELEMENT_NODE:
          xnode = convert((Element)child,xdoc);
          break;
        case Node.TEXT_NODE:
          xnode = convert((Text)child,xdoc);
          break;
        case Node.COMMENT_NODE:
        case Node.PROCESSING_INSTRUCTION_NODE:
        case Node.CDATA_SECTION_NODE:
          System.out.println("(DOMToModel)Not implemented node type: " + type);
      }

      if (xnode != null)
        xel.appendChild(xnode);

      child = child.getNextSibling();
    }

    return xel;
  }

  // uri is the namespace, raw is the qualified name. ex) p:part
  // Returns null if the prefix is null or the uri is an empty string.
  private XNamespace getNamespace(String uri, String raw) {
    String prefix = getPrefix(raw);
    if (prefix == null || uri.length() == 0)
      return null;
    else if (xnamespaces.containsKey(prefix)) { // namespaces is Hashtable
      return (XNamespace)xnamespaces.get(prefix);
    }
    else {
      XNamespace xns = new XNamespace(prefix,uri);
      xnamespaces.put(prefix,uri);
      return xns;
    }
  }

  private String getPrefix(String raw) {
    if (raw.indexOf(":") == -1)
      return null; // no prefix
    else
      return raw.substring(0,raw.indexOf(":"));
  }

  private XText convert(Text text, XDocument xdoc)  throws XModelException {
    String value = text.getNodeValue();
    XText xtext = xdoc.createText(value);
    return xtext;
  }

  private XComment convert(Comment comment, XDocument xdoc) {
    // TBD
    return null;
  }

  // TBD - Implement remaining convert() methods.
/*
  private XNode convert(Node node, XDocument xdoc) {
    short type = node.getNodeType();
    switch (type) {
      case Node.DOCUMENT_NODE:
      case Node.DOCUMENT_TYPE_NODE:
      case Node.ELEMENT_NODE:
      case Node.ENTITY_REFERENCE_NODE:
      case Node.CDATA_SECTION_NODE:
      case Node.TEXT_NODE:
      case Node.PROCESSING_INSTRUCTION_NODE:
        System.out.println("(DOMToModel)Not implemented node type: " + type);
    }
    return null;
  }
*/
}