/*
 * (C) Copyright Bora Software Inc. 2002, 2003 - All Rights Reserved.
 *
 * $Id: ModelLoader.java,v 1.9 2002/11/02 18:11:28 kboo Exp $
 */
package com.bs.xdbms.persistence;

import java.sql.*;
import java.util.Vector;
import com.bs.xdbms.datamodel.*;

public class ModelLoader {
  // TBD - review connection synchronization issue for concurrent use
  protected Connection connection;

  public ModelLoader(Connection conn) throws SQLException {
    connection = conn;
  }

  public void commit() {
    try {
      connection.commit();
    } catch (SQLException e) {
      // nothing to do
    }
  }

  public String[] loadDocumentNames() throws SQLException, XModelException {
    Vector v = new Vector();
    String sql = "SELECT X_DOCUMENT.BASE_URI FROM X_DOCUMENT";
    Statement stmt = connection.createStatement();
    ResultSet rs = stmt.executeQuery(sql);
    while (rs.next()) {
      v.addElement(rs.getString(1));
    }
    stmt.close();

    String[] docNames = new String[v.size()];
    for (int i=0; i<v.size(); i++)
      docNames[i] = (String)v.elementAt(i);

    return docNames;
  }

  public XDocument loadDocument(String baseURI) 
      throws SQLException, XModelException {
    XDocument xdoc = new XDocument();
    xdoc.setBaseURI(baseURI);
    long oid = -1;
    String sql = "SELECT X_DOCUMENT.X_DOCUMENT_OID FROM X_DOCUMENT ";
    sql += "WHERE X_DOCUMENT.BASE_URI = ?";
    PreparedStatement pstmt = connection.prepareStatement(sql);
    pstmt.setString(1,baseURI);
    ResultSet rs = pstmt.executeQuery();
    while (rs.next()) {
      // TBD - I am expecting to have one row as the result.
      // For this, baseURI must be unique.
      oid = rs.getLong(1);
      xdoc.setOID(oid);
    }
    pstmt.close();

    // Handle children
    sql = "SELECT CHILD_NODE_OID, CHILD_NODE_KIND ";
    sql += "FROM PC_SEQUENCE WHERE PC_SEQUENCE.PARENT_NODE_OID = ? ";
    sql += "AND PC_SEQUENCE.PARENT_NODE_KIND = ?";
    pstmt = connection.prepareStatement(sql);
    pstmt.setLong(1,oid);
    pstmt.setString(2,"document");
    rs = pstmt.executeQuery();
    String nodeKind;
    XNode node;
    long childOID = -1;
    while (rs.next()) {
      childOID = rs.getLong(1);
      nodeKind = rs.getString(2);
      node = loadNode(childOID,nodeKind,xdoc);
      // Assert - nodeKind is a valid kind as a doc's child.
      xdoc.appendChild(node);
    }
    pstmt.close();
    return xdoc;
  }

  protected XNode loadNode
                  (long oid, 
                   String nodeKind, 
                   XDocument xdoc)
      throws SQLException, XModelException {
    if (nodeKind.equals("element")) 
      return loadElement(oid,xdoc);
    else if (nodeKind.equals("processing-instruction"))
      return loadPI(oid,xdoc);
    else if (nodeKind.equals("comment")) 
      return loadComment(oid,xdoc);
    else if (nodeKind.equals("attribute")) 
      return loadAttribute(oid,xdoc);
    else if (nodeKind.equals("namespace")) 
      return loadNamespace(oid,xdoc);
    else if (nodeKind.equals("text")) 
      return loadText(oid,xdoc);
    else
      return null; // error condition
  }

  private XElement loadElement(long oid, XDocument xdoc) 
      throws SQLException, XModelException {
    String nsURI;
    String localName;
    String baseURI;
    String stringValue;
    String uniqueId;
    String sql = "SELECT * FROM X_ELEMENT WHERE X_ELEMENT_OID = ?";
    PreparedStatement pstmt = connection.prepareStatement(sql);
    pstmt.setLong(1,oid);
    ResultSet rs = pstmt.executeQuery();

    XElement xel = null;
    while (rs.next()) {
      nsURI = rs.getString(2);
      localName = rs.getString(3);
      baseURI = rs.getString(4);
      stringValue = rs.getString(5);
      uniqueId = rs.getString(5);
      xel = xdoc.createElement(nsURI,localName);
      xel.setBaseURI(baseURI);
      xel.setStringValue(stringValue);
      xel.setUniqueID(uniqueId);
    }
    pstmt.close();

    // Handle children
    sql = "SELECT CHILD_NODE_OID, CHILD_NODE_KIND ";
    sql += "FROM PC_SEQUENCE WHERE PC_SEQUENCE.PARENT_NODE_OID = ? ";
    sql += "AND PC_SEQUENCE.PARENT_NODE_KIND = ?";
    pstmt = connection.prepareStatement(sql);
    pstmt.setLong(1,oid);
    pstmt.setString(2,"element");
    rs = pstmt.executeQuery();
    String nodeKind;
    XNode node;
    long childOID = -1;
    while (rs.next()) {
      childOID = rs.getLong(1);
      nodeKind = rs.getString(2);
      node = loadNode(childOID,nodeKind,xdoc);
      // Assert - nodeKind is a valid kind as an element's child.
      xel.appendChild(node);
    }
    pstmt.close();

    // Handle attributes
    sql = "SELECT * FROM ATTR_SEQUENCE WHERE X_ELEMENT_OID = ? ";
    pstmt = connection.prepareStatement(sql);
    pstmt.setLong(1,oid);
    rs = pstmt.executeQuery();
    nodeKind = "attribute";
    long attrOID = -1;
    while (rs.next()) {
      attrOID = rs.getLong(2);
      node = loadNode(attrOID,nodeKind,xdoc);
      //xel.appendChild(node);
      xel.addAttribute((XAttribute)node);
    }
    pstmt.close();

    // Handle namespaces
    sql = "SELECT * FROM NS_SEQUENCE WHERE X_ELEMENT_OID = ? ";
    pstmt = connection.prepareStatement(sql);
    pstmt.setLong(1,oid);
    rs = pstmt.executeQuery();
    nodeKind = "namespace";
    long nsOID = -1;
    while (rs.next()) {
      nsOID = rs.getLong(2);
      node = loadNode(nsOID,nodeKind,xdoc);
      //xel.appendChild(node);
      xel.addNamespace((XNamespace)node);
    }
    pstmt.close();

    return xel;
  }

  private XProcessingInstruction loadPI(long oid, XDocument xdoc) 
      throws SQLException {
    String nsURI;
    String localName;
    String stringValue;
    String sql = "SELECT * FROM X_PI WHERE X_PI_OID = ?";
    PreparedStatement pstmt = connection.prepareStatement(sql);
    pstmt.setLong(1,oid);
    ResultSet rs = pstmt.executeQuery();

    XProcessingInstruction xpi = null;
    while (rs.next()) {
      nsURI = rs.getString(2);
      localName = rs.getString(3);
      stringValue = rs.getString(4);
      xpi = xdoc.createProcessingInstruction(nsURI,localName);
      xpi.setStringValue(stringValue);
    }
    pstmt.close();
    return xpi;
  }

  private XComment loadComment(long oid, XDocument xdoc) throws SQLException {
    String stringValue;
    String sql = "SELECT * FROM X_COMMENT WHERE X_COMMENT_OID = ?";
    PreparedStatement pstmt = connection.prepareStatement(sql);
    pstmt.setLong(1,oid);
    ResultSet rs = pstmt.executeQuery();

    XComment xcomm = null;
    while (rs.next()) {
      stringValue = rs.getString(2);
      xcomm = xdoc.createComment(stringValue);
    }
    pstmt.close();
    return xcomm;
  }

  private XAttribute loadAttribute(long oid, XDocument xdoc) 
      throws SQLException {
    String nsURI;
    String localName;
    String stringValue;
    String sql = "SELECT * FROM X_ATTRIBUTE WHERE X_ATTRIBUTE_OID = ?";
    PreparedStatement pstmt = connection.prepareStatement(sql);
    pstmt.setLong(1,oid);
    ResultSet rs = pstmt.executeQuery();

    XAttribute xattr = null;
    while (rs.next()) {
      nsURI = rs.getString(2);
      localName = rs.getString(3);
      stringValue = rs.getString(4);
      xattr = xdoc.createAttribute(nsURI,localName,stringValue);
    }
    pstmt.close();
    return xattr;
  }

  private XNamespace loadNamespace(long oid, XDocument xdoc) 
      throws SQLException {
    String nsURI;
    String localName;
    String stringValue;
    String sql = "SELECT * FROM X_NAMESPACE WHERE X_NAMESPACE_OID = ?";
    PreparedStatement pstmt = connection.prepareStatement(sql);
    pstmt.setLong(1,oid);
    ResultSet rs = pstmt.executeQuery();

    XNamespace xns = null;
    while (rs.next()) {
      localName = rs.getString(3); // namespace prefix
      stringValue = rs.getString(4); // namespace uri
      xns = new XNamespace(localName,stringValue);
    }
    pstmt.close();
    return xns;
  }

  private XText loadText(long oid, XDocument xdoc) throws SQLException {
    String stringValue;
    String sql = "SELECT * FROM X_TEXT WHERE X_TEXT_OID = ?";
    PreparedStatement pstmt = connection.prepareStatement(sql);
    pstmt.setLong(1,oid);
    ResultSet rs = pstmt.executeQuery();

    XText xtext = null;
    while (rs.next()) {
      stringValue = rs.getString(2);
      xtext = xdoc.createText(stringValue);
    }
    pstmt.close();
    return xtext;
  }

}
