/*
 * (C) Copyright Bora Software Inc. 2002, 2003 - All Rights Reserved.
 *
 * $Id: OIDManager.java,v 1.6 2002/11/16 23:10:30 kboo Exp $
 */
package com.bs.xdbms.persistence;

import java.sql.*;

public final class OIDManager {
  // TBD - review connection synchronization issue for concurrent use
  private Connection connection;

  private final String NEXT_OID = "NEXT_OID"; // table name

  // column names
  private final String X_DOCUMENT_OID  = "X_DOCUMENT_OID";
  private final String X_NAMESPACE_OID = "X_NAMESPACE_OID";
  private final String X_ELEMENT_OID   = "X_ELEMENT_OID";
  private final String X_ATTRIBUTE_OID = "X_ATTRIBUTE_OID";
  private final String X_PI_OID        = "X_PI_OID";
  private final String X_COMMENT_OID   = "X_COMMENT_OID";
  private final String X_TEXT_OID      = "X_TEXT_OID";
  
  private final String XDB_COLLECTION_OID = "XDB_COLLECTION_OID";

  public OIDManager(Connection conn) throws SQLException {
    connection = conn;
    if (connection.getAutoCommit())
      connection.setAutoCommit(false);
  }

  public void commit() {
    try {
      connection.commit();
    } catch (SQLException e) {
      // nothing to do
    }
  }

  public void rollback() {
    try {
      connection.rollback();
    } catch (SQLException e) {
      // nothing to do
    }
  }
  
  // TBD - Recycle deleted OIDs
  public long getUniqueOID(String nodeKind) throws SQLException {
    long oid = getCurrentOID(nodeKind);
    incrementOID(nodeKind,oid+1);
    return oid;
  }

  private void incrementOID(String nodeKind, long oid) throws SQLException {
    String sql = "UPDATE NEXT_OID SET ";
    sql += getColumnName(nodeKind);
    sql += " = ? WHERE NEXT_OID_OID = 0";
    PreparedStatement pstmt = connection.prepareStatement(sql);
    pstmt.setLong(1,oid);
    pstmt.executeUpdate();
    pstmt.close();
  }

  private long getCurrentOID(String nodeKind) throws SQLException {
    String sql = "SELECT ";
    sql += getColumnName(nodeKind);
    sql += " FROM NEXT_OID FOR UPDATE";
    Statement stmt = connection.createStatement();
    ResultSet rs = stmt.executeQuery(sql);

    long oid = -1;
    while (rs.next()) {
      oid = rs.getLong(getColumnName(nodeKind));
      //System.out.println(nodeKind + " oid is: " + rs.getObject(1));
      // TBD - Can I not increment the value here?
      stmt.close();
      return oid;
    }

    stmt.close();
    return oid; // exceptional condition
  }

  private String getColumnName(String nodeKind) {
    if (nodeKind.equals("document"))
      return X_DOCUMENT_OID;
    else if (nodeKind.equals("namespace"))
      return X_NAMESPACE_OID;
    else if (nodeKind.equals("element"))
      return X_ELEMENT_OID;
    else if (nodeKind.equals("attribute"))
      return X_ATTRIBUTE_OID;
    else if (nodeKind.equals("processing-instruction"))
      return X_PI_OID;
    else if (nodeKind.equals("comment"))
      return X_COMMENT_OID;
    else if (nodeKind.equals("text"))
      return X_TEXT_OID;
    else if (nodeKind.equals("collection")) // this is not a node - exceptional case
      return XDB_COLLECTION_OID;
    else
      return null; // exceptional condition
  }
}
