package com.bs.xdbms.util;

import java.sql.*;
import java.util.Vector;

import com.bs.xdbms.datamodel.*;
import com.bs.xdbms.persistence.DBUpdater;

import org.xmldb.api.base.Collection;
import org.xmldb.api.base.XMLDBException;

public class ExtendedDBUpdater extends DBUpdater {

	public ExtendedDBUpdater(Connection conn) throws SQLException {
		super(conn);
	}

    public void insert(Collection collection, XDocument xdoc) 
        throws SQLException, XMLDBException {
  	  namespaces = new Vector();
      long oid = getNewXDocumentOID();
      long collectionOID = 
        stringToLong(collection.getProperty(Constants.XDB_COLLECTION_OID));
      xdoc.setOID(oid);
      String baseURI = xdoc.getBaseURI();
      String sql = 
        "INSERT INTO X_DOCUMENT (X_DOCUMENT_OID,BASE_URI,XDB_COLLECTION_OID) VALUES (?,?,?)";
      PreparedStatement pstmt = connection.prepareStatement(sql);
      pstmt.setLong(1,oid);
      pstmt.setString(2,baseURI);
      pstmt.setLong(3,collectionOID);
      pstmt.executeUpdate();
      pstmt.close();

      // Setting relationships with children
      Vector children = xdoc.getChildren();
      XNode child;
      for (int i=0; i<children.size(); i++) {
        child = (XNode)children.elementAt(i);
        if (child.getNodeKind().equals("element")) {
          insert((XElement)child);
        } else if (child.getNodeKind().equals("processing-instruction")) {
          insert((XProcessingInstruction)child);
        } else if (child.getNodeKind().equals("comment")) {
          insert((XComment)child);
        } else
          ; // TBD - Need to raise XModelException ?
        insertPCSequence(xdoc,child);
      } // end for
    }
    
  private long stringToLong(String s) {
    return (new Long(s)).longValue();
  }
}
