package com.bs.xdbms.util;

import java.sql.*;
import java.util.Vector;

import com.bs.xdbms.datamodel.*;
import com.bs.xdbms.persistence.ModelLoader;

import org.xmldb.api.base.Collection;
import org.xmldb.api.base.XMLDBException;

public class ExtendedModelLoader extends ModelLoader {

	public ExtendedModelLoader(Connection conn) throws SQLException {
		super(conn);
	}
	
	public XDocument loadDocument(Collection collection, String baseURI) 
        throws SQLException, XModelException, XMLDBException {
    XDocument xdoc = null;
    //xdoc.setBaseURI(baseURI);
    long oid = -1;
    long collectionOID = 
      stringToLong(collection.getProperty(Constants.XDB_COLLECTION_OID));
    String sql = "SELECT X_DOCUMENT.X_DOCUMENT_OID FROM X_DOCUMENT ";
    sql += "WHERE X_DOCUMENT.BASE_URI = ? AND X_DOCUMENT.XDB_COLLECTION_OID = ?";
    PreparedStatement pstmt = connection.prepareStatement(sql);
    pstmt.setString(1,baseURI);
    pstmt.setLong(2,collectionOID);
    ResultSet rs = pstmt.executeQuery();
    while (rs.next()) {
      // TBD - I am expecting to have one row as the result.
      // For this, baseURI must be unique.
      if (xdoc == null) {
      	xdoc = new XDocument();
      	xdoc.setBaseURI(baseURI);
      }
      oid = rs.getLong(1);
      xdoc.setOID(oid);
    }
    pstmt.close();

    // Handle children
    sql = "SELECT CHILD_NODE_OID, CHILD_NODE_KIND ";
    sql += "FROM PC_SEQUENCE WHERE PC_SEQUENCE.PARENT_NODE_OID = ? ";
    sql += "AND PC_SEQUENCE.PARENT_NODE_KIND = ?";
    pstmt = connection.prepareStatement(sql);
    pstmt.setLong(1,oid);
    pstmt.setString(2,"document");
    rs = pstmt.executeQuery();
    String nodeKind;
    XNode node;
    long childOID = -1;
    while (rs.next()) {
      childOID = rs.getLong(1);
      nodeKind = rs.getString(2);
      node = loadNode(childOID,nodeKind,xdoc);
      // Assert - nodeKind is a valid kind as a doc's child.
      xdoc.appendChild(node);
    }
    pstmt.close();
    return xdoc;
  }

  public String[] loadDocumentNames(Collection collection) 
      throws SQLException, XModelException, XMLDBException {
    Vector v = new Vector();
    String sql = "SELECT X_DOCUMENT.BASE_URI FROM X_DOCUMENT ";
    sql += "WHERE X_DOCUMENT.XDB_COLLECTION_OID = ?";
    PreparedStatement pstmt = connection.prepareStatement(sql);
    long collectionOID = 
      stringToLong(collection.getProperty(Constants.XDB_COLLECTION_OID));
    pstmt.setLong(1,collectionOID);
    ResultSet rs = pstmt.executeQuery();    
    while (rs.next()) {
      v.addElement(rs.getString(1));
    }
    pstmt.close();

    String[] docNames = new String[v.size()];
    for (int i=0; i<v.size(); i++)
      docNames[i] = (String)v.elementAt(i);

    return docNames;
  }
  
  private long stringToLong(String s) {
    return (new Long(s)).longValue();
  }
}
