/*
 * (C) Copyright Bora Software Inc. 2002, 2003 - All Rights Reserved.
 *
 * $Id: ResourceImpl.java,v 1.3 2002/12/13 22:37:48 kboo Exp $
 */

package com.bs.xdbms.xmldb.base;

import org.xmldb.api.base.Collection;
import org.xmldb.api.base.Resource;
import org.xmldb.api.base.XMLDBException;

/**
 * <code>Resource</code> is a container for data stored within the database. Raw resources
 * are not particulary useful. It is necessary to have a resource implementation
 * that provides handling for a specific content type before anything useful can
 * be done.
 */
public class ResourceImpl implements Resource {      

  protected String id;
  protected String type;
  protected Collection parentCollection;

  protected ResourceImpl() {
  }

  public ResourceImpl(String resourceId, String resourceType, Collection parent) {
    id = resourceId; // id is null for an anonymous resource
    type = resourceType;
    parentCollection = parent;
  }

   /**
    * Returns the <code>Collection</code> instance that this resource is 
    * associated with. All resources must exist within the context of a 
    * <code>collection</code>.
    *
    * @return the collection associated with the resource.
    * @exception XMLDBException with expected error codes.<br />
    *  <code>ErrorCodes.VENDOR_ERROR</code> for any vendor
    *  specific errors that occur.<br /> 
    */
  public Collection getParentCollection() throws XMLDBException {
    return parentCollection;
  }

   /**
    * Returns the unique id for this <code>Resource</code> or null if the 
    * <code>Resource</code> is anonymous. The <code>Resource</code> will be 
    * anonymous if it is obtained as the result of a query.
    *
    * @return the id for the Resource or null if no id exists.
    * @exception XMLDBException with expected error codes.<br />
    *  <code>ErrorCodes.VENDOR_ERROR</code> for any vendor
    *  specific errors that occur.<br /> 
    */
  public String getId() throws XMLDBException {
    return id;
  }

   /**
    * Returns the resource type for this Resource. 
    * <p />
    * XML:DB defined resource types are: <p />
    * XMLResource - all XML data stored in the database<br />
    * BinaryResource - Binary blob data stored in the database<br />
    * 
    * @return the resource type for the Resource.
    * @exception XMLDBException with expected error codes.<br />
    *  <code>ErrorCodes.VENDOR_ERROR</code> for any vendor
    *  specific errors that occur.<br /> 
    */
  public String getResourceType() throws XMLDBException {
    return type;
  }
   
   /**
    * Retrieves the content from the resource. The type of the content varies
    * depending what type of resource is being used.
    *
    * @return the content of the resource.
    * @exception XMLDBException with expected error codes.<br />
    *  <code>ErrorCodes.VENDOR_ERROR</code> for any vendor
    *  specific errors that occur.<br /> 
    */
  public Object getContent() throws XMLDBException {
    // TBD
    return null;
  }

   /**
    * Sets the content for this resource. The type of content that can be set
    * depends on the type of resource being used.
    *
    * @param value the content value to set for the resource.
    * @exception XMLDBException with expected error codes.<br />
    *  <code>ErrorCodes.VENDOR_ERROR</code> for any vendor
    *  specific errors that occur.<br /> 
    */
  public void setContent(Object value) throws XMLDBException {
    // TBD
  }
}

