/*
 * (C) Copyright Bora Software Inc. 2002, 2003 - All Rights Reserved.
 *
 * $Id: ServiceImpl.java,v 1.3 2002/11/02 18:11:28 kboo Exp $
 */
package com.bs.xdbms.xmldb.base;

import java.util.Vector;

import org.xmldb.api.base.Service;
import org.xmldb.api.base.Collection;
import org.xmldb.api.base.XMLDBException;

/**
 * The <code>Service</code> interface provides an extension mechanism for 
 * <code>Collection</code> implementations. It is to be implented by Service
 * instances that define their own set of methods to perform the necessary
 * action. For an example of what a functional <code>Service</code> interface
 * should look like look at XPathQueryService.
 *
 * @see org.xmldb.api.modules.XPathQueryService
 */
public class ServiceImpl extends ConfigurableImpl implements Service {
	
	protected Collection collection = null;

   /**
    * Returns the name associated with the Service instance.
    *
    * @return the name of the object.
    * @exception XMLDBException with expected error codes.<br />
    *  <code>ErrorCodes.VENDOR_ERROR</code> for any vendor
    *  specific errors that occur.<br />
    */
   public String getName() throws XMLDBException {
     return "GenericService";
   }
   
   /**
    * Gets the Version attribute of the Service object
    *
    * @return The Version value
    * @exception XMLDBException with expected error codes.<br />
    *  <code>ErrorCodes.VENDOR_ERROR</code> for any vendor
    *  specific errors that occur.<br /> 
    */
   public String getVersion() throws XMLDBException {
     return "0.1";
   }

   /**
    * Sets the Collection attribute of the Service object
    *
    * @param col The new Collection value
    * @exception XMLDBException with expected error codes.<br />
    *  <code>ErrorCodes.VENDOR_ERROR</code> for any vendor
    *  specific errors that occur.<br />
    */
   public void setCollection(Collection col) throws XMLDBException {
     collection = col;
   }
}

