/*
 * (C) Copyright Bora Software Inc. 2002, 2003 - All Rights Reserved.
 *
 * $Id: XMLResourceImpl.java,v 1.5 2002/12/13 22:37:48 kboo Exp $
 */
package com.bs.xdbms.xmldb.modules;

import org.xmldb.api.base.*;
import org.xmldb.api.modules.XMLResource;

import com.bs.xdbms.xmldb.base.ResourceImpl;

import org.w3c.dom.*;
import org.xml.sax.*;

/**
 * Provides access to XML resources stored in the database. An XMLResource can
 * be accessed either as text XML or via the DOM or SAX APIs.<p />
 *
 * The default behavior for getContent and setContent is to work with XML data
 * as text so these methods work on <code>String</code> content.
 */
final public class XMLResourceImpl extends ResourceImpl implements XMLResource {

  private Node content;
 
  public XMLResourceImpl(String resourceId, Collection parent) {
    id = resourceId;
    type = XMLResource.RESOURCE_TYPE;
    parentCollection = parent;
  }

   /**
    * Returns the unique id for the parent document to this <code>Resource</code>
    * or null if the <code>Resource</code> does not have a parent document. 
    * <code>getDocumentId()</code> is typically used with <code>Resource</code>
    * instances retrieved using a query. It enables accessing the parent
    * document of the <code>Resource</code> even if the <code>Resource</code> is
    * a child node of the document. If the <code>Resource</code> was not
    * obtained through a query then <code>getId()</code> and 
    * <code>getDocumentId()</code> will return the same id.
    *
    * @return the id for the parent document of this <code>Resource</code> or 
    *  null if there is no parent document for this <code>Resource</code>.
    * @exception XMLDBException with expected error codes.<br />
    *  <code>ErrorCodes.VENDOR_ERROR</code> for any vendor
    *  specific errors that occur.<br /> 
    */
   public String getDocumentId() throws XMLDBException {
     // TBD
     return id;
   }
   
   /**
    * Returns the content of the <code>Resource</code> as a DOM Node.
    *
    * @return The XML content as a DOM <code>Node</code>
    * @exception XMLDBException with expected error codes.<br />
    *  <code>ErrorCodes.VENDOR_ERROR</code> for any vendor
    *  specific errors that occur.<br />
    */
   public Node getContentAsDOM() throws XMLDBException {
     return content;
   }

   /**
    * Sets the content of the <code>Resource</code> using a DOM Node as the
    * source.
    *
    * @param content The new content value
    * @exception XMLDBException with expected error codes.<br />
    *  <code>ErrorCodes.VENDOR_ERROR</code> for any vendor
    *  specific errors that occur.<br />
    *  <code>ErrorCodes.INVALID_RESOURCE</code> if the content value provided is
    *  null.<br />
    *  <code>ErrorCodes.WRONG_CONTENT_TYPE</code> if the content provided in not
    *  a valid DOM <code>Node</code>.
    */
   public void setContentAsDOM(Node content) throws XMLDBException {
     // TBD
     this.content = content;
   }

   /**
    * Allows you to use a <code>ContentHandler</code> to parse the XML data from
    * the database for use in an application.
    *
    * @param handler the SAX <code>ContentHandler</code> to use to handle the
    *  <code>Resource</code> content.
    * @exception XMLDBException with expected error codes.<br />
    *  <code>ErrorCodes.VENDOR_ERROR</code> for any vendor
    *  specific errors that occur.<br />
    *  <code>ErrorCodes.INVALID_RESOURCE</code> if the 
    *  <code>ContentHandler</code> provided is null.<br />
    */
   public void getContentAsSAX(ContentHandler handler) throws XMLDBException {
     // TBD
   }

   /**
    * Sets the content of the <code>Resource</code> using a SAX 
    * <code>ContentHandler</code>.
    *
    * @return a SAX <code>ContentHandler</code> that can be used to add content
    *  into the <code>Resource</code>.
    * @exception XMLDBException with expected error codes.<br />
    *  <code>ErrorCodes.VENDOR_ERROR</code> for any vendor
    *  specific errors that occur.<br />
    */
   public ContentHandler setContentAsSAX() throws XMLDBException {
     // TBD
     return null;
   }
}